#include <clippy/clippy.hpp>
#include <clippy/target.hpp>

#include <utils/parametres.hpp>
#include <utils/config_path.hpp>

#include <iostream>
#include <memory>
#include <vector>
#include <filesystem>

void Clippy::Run(const std::vector<std::string>& args) {
  if (auto result = TryExecuteClippyCommand(args); result) {
    result->Execute();
    return;
  }
  if (auto result = GetScriptTarget(args); result) {
    result->Execute();
    return;
  }
}

Clippy::TargetPtr Clippy::TryExecuteClippyCommand(
    const std::vector<std::string>& args) {
  using namespace utils::parametres;
  using namespace clippy::targets;
  if (CheckPatternParametres(args, Parameter::Skip, "help",
                             Parameter::Anything)) {
    std::cout << "Hello I'm clippy" << std::endl;
    std::cout << "Parametres: { ";
    for (size_t i = 0; i < args.size(); ++i) {
      std::cout << args[i] << (i + 1 == args.size() ? "" : ",") << " ";
    }
    std::cout << "}" << std::endl;

    return std::make_unique<EmptyTarget>();
  }

  if (CheckPatternParametres(args, Parameter::Skip, "cfg",
                             Parameter::Anything)) {
    LoadProjects();
    auto p = projects_->GetCurrentProject();

    if (p.has_value()) {
      return std::make_unique<OpenProjectConfig>(p->GetConfig());
    } else {
      return std::make_unique<CreateProjectConfig>(projects_.value());
    }
  }

  if (CheckPatternParametres(args, Parameter::Skip, "crt",
                             Parameter::Anything)) {
    LoadProjects();
    return std::make_unique<CreateProjectConfig>(projects_.value());
  }

  return nullptr;
}

Clippy::TargetPtr Clippy::GetScriptTarget(
    const std::vector<std::string>& args) {
  LoadProjects();
  auto p = projects_->GetCurrentProject();

  if (!p.has_value() || args.size() < 2) {
    return nullptr;
  }

  auto cfg = p->GetConfig();
  return cfg.GetTarget(args[1]);
}

void Clippy::LoadProjects() {
  if (!projects_.has_value()) {
    projects_.emplace(utils::GetProjectDirectory() / "projects");
  }
}
